@@ -20,15 +20,18 @@ are what gets used.
 
 from array import array
 from itertools import count
-import re, time
+import re
+import sys
+import time
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
-# Sadly the name "constraints" conflicts with many of the function
-# arguments in this module, so we rename it:
+import weakref
 from formencode import compound, validators
 from classregistry import findClass
+# Sadly the name "constraints" conflicts with many of the function
+# arguments in this module, so we rename it:
 import constraints as constrs
 import sqlbuilder
 from styles import capword
@@ -106,6 +109,7 @@ class SOCol(object):
                  title=None,
                  tags=[],
                  origName=None,
+                 dbEncoding=None,
                  extra_vars=None):
 
         super(SOCol, self).__init__()
@@ -197,6 +201,9 @@ class SOCol(object):
         if validator: _validators.append(validator)
         if validator2: _validators.insert(0, validator2)
         _vlen = len(_validators)
+        if _vlen:
+            for _validator in _validators:
+                _validator.soCol=weakref.proxy(self)
         if _vlen == 0:
             self.validator = None # Set sef.{from,to}_python
         elif _vlen == 1:
@@ -210,6 +217,7 @@ class SOCol(object):
         self.origName = origName or name
         self.title = title
         self.tags = tags
+        self.dbEncoding = dbEncoding
 
         if extra_vars:
             for name, value in extra_vars.items():
@@ -368,6 +376,22 @@ class SOCol(object):
     def __delete__(self, obj):
         raise AttributeError("I can't be deleted from %r" % obj)
 
+    def getDbEncoding(self, state, default='utf-8'):
+        if self.dbEncoding:
+            return self.dbEncoding
+        dbEncoding = state.soObject.sqlmeta.dbEncoding
+        if dbEncoding:
+            return dbEncoding
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            dbEncoding = None
+        else:
+            dbEncoding = getattr(connection, "dbEncoding", None)
+        if not dbEncoding:
+            dbEncoding = default
+        return dbEncoding
+
 
 class Col(object):
 
@@ -410,6 +434,13 @@ class Col(object):
             self._name or '(unnamed)')
 
 
+class SOValidator(validators.Validator):
+    def getDbEncoding(self, state, default='utf-8'):
+        try:
+            return self.dbEncoding
+        except AttributeError:
+            return self.soCol.getDbEncoding(state, default=default)
+
 
 class SOStringLikeCol(SOCol):
     """A common ancestor for SOStringCol and SOUnicodeCol"""
@@ -474,7 +505,7 @@ class SOStringLikeCol(SOCol):
             if self.connection and self.connection.can_use_max_types():
                 type = 'VARCHAR(MAX)'
             else:
-                type = 'varchar(4000)'
+                type = 'VARCHAR(4000)'
         elif self.varchar:
             type = 'VARCHAR(%i)' % self.length
         else:
@@ -498,7 +529,7 @@ class SOStringLikeCol(SOCol):
             return self._sqlType()
 
 
-class StringValidator(validators.Validator):
+class StringValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
@@ -507,10 +538,8 @@ class StringValidator(validators.Validator):
             connection = state.connection or state.soObject._connection
             binaryType = connection._binaryType
         except AttributeError:
-            dbEncoding = "ascii"
             binaryType = type(None) # Just a simple workaround
-        else:
-            dbEncoding = getattr(connection, "dbEncoding", None) or "ascii"
+        dbEncoding = self.getDbEncoding(state, default='ascii')
         if isinstance(value, unicode):
             return value.encode(dbEncoding)
         if self.dataType and isinstance(value, self.dataType):
@@ -533,13 +562,18 @@ class SOStringCol(SOStringLikeCol):
 class StringCol(Col):
     baseClass = SOStringCol
 
-class UnicodeStringValidator(validators.Validator):
 
-    def getDbEncoding(self, state):
-        try:
-            return self.dbEncoding
-        except AttributeError:
-            return self.soCol.getDbEncoding(state)
+class NQuoted(sqlbuilder.SQLExpression):
+    def __init__(self, value):
+        assert isinstance(value, unicode)
+        self.value = value
+    def __hash__(self):
+        return hash(self.value)
+    def __sqlrepr__(self, db):
+        assert db == 'mssql'
+        return "N" + sqlbuilder.sqlrepr(self.value, db)
+
+class UnicodeStringValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
@@ -561,6 +595,13 @@ class UnicodeStringValidator(validators.Validator):
         if isinstance(value, (str, sqlbuilder.SQLExpression)):
             return value
         if isinstance(value, unicode):
+            try:
+                connection = state.connection or state.soObject._connection
+            except AttributeError:
+                pass
+            else:
+                if connection.dbName == 'mssql':
+                    return NQuoted(value)
             return value.encode(self.getDbEncoding(state))
         if hasattr(value, '__unicode__'):
             return unicode(value).encode(self.getDbEncoding(state))
@@ -568,35 +609,20 @@ class UnicodeStringValidator(validators.Validator):
             (self.name, type(value), value), value, state)
 
 class SOUnicodeCol(SOStringLikeCol):
-    def __init__(self, **kw):
-        self.dbEncoding = kw.pop('dbEncoding', None)
-        super(SOUnicodeCol, self).__init__(**kw)
+    def _mssqlType(self):
+        if self.customSQLType is not None:
+            return self.customSQLType
+        return 'N' + super(SOUnicodeCol, self)._mssqlType()
 
     def createValidators(self):
-        return [UnicodeStringValidator(name=self.name, soCol=self)] + \
+        return [UnicodeStringValidator(name=self.name)] + \
             super(SOUnicodeCol, self).createValidators()
 
-    def getDbEncoding(self, state):
-        if self.dbEncoding:
-            return self.dbEncoding
-        dbEncoding = state.soObject.sqlmeta.dbEncoding
-        if dbEncoding:
-            return dbEncoding
-        try:
-            connection = state.connection or state.soObject._connection
-        except AttributeError:
-            dbEncoding = None
-        else:
-            dbEncoding = getattr(connection, "dbEncoding", None)
-        if not dbEncoding:
-            dbEncoding = "utf-8"
-        return dbEncoding
-
 class UnicodeCol(Col):
     baseClass = SOUnicodeCol
 
 
-class IntValidator(validators.Validator):
+class IntValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
@@ -677,7 +703,7 @@ class BigIntCol(Col):
     baseClass = SOBigIntCol
 
 
-class BoolValidator(validators.Validator):
+class BoolValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
@@ -724,7 +750,7 @@ class BoolCol(Col):
     baseClass = SOBoolCol
 
 
-class FloatValidator(validators.Validator):
+class FloatValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
@@ -936,10 +962,13 @@ class ForeignKey(KeyCol):
         super(ForeignKey, self).__init__(foreignKey=foreignKey, **kw)
 
 
-class EnumValidator(validators.Validator):
+class EnumValidator(SOValidator):
 
     def to_python(self, value, state):
         if value in self.enumValues:
+            if isinstance(value, unicode):
+                dbEncoding = self.getDbEncoding(state)
+                value = value.encode(dbEncoding)
             return value
         elif not self.notNone and value is None:
             return None
@@ -1009,7 +1038,7 @@ class EnumCol(Col):
     baseClass = SOEnumCol
 
 
-class SetValidator(validators.Validator):
+class SetValidator(SOValidator):
     """
     Translates Python tuples into SQL comma-delimited SET strings.
     """
@@ -1071,11 +1100,23 @@ class DateTimeValidator(validators.DateValidator):
                 else:
                     return datetime.time(value.hour, value.minute, int(value.second))
         try:
-            stime = time.strptime(value, self.format)
+            if self.format.find(".%f") >= 0:
+                if '.' in value:
+                    _value = value.split('.')
+                    microseconds = _value[-1]
+                    _l = len(microseconds)
+                    if _l < 6:
+                        _value[-1] = microseconds + '0'*(6 - _l)
+                    elif _l > 6:
+                        _value[-1] = microseconds[:6]
+                    if _l != 6:
+                        value = '.'.join(_value)
+                else:
+                    value += '.0'
+            return datetime.datetime.strptime(value, self.format)
         except:
             raise validators.Invalid("expected a date/time string of the '%s' format in the DateTimeCol '%s', got %s %r instead" % \
                 (self.format, self.name, type(value), value), value, state)
-        return datetime.datetime(*stime[:6])
 
     def from_python(self, value, state):
         if value is None:
@@ -1102,11 +1143,25 @@ if mxdatetime_available:
             elif isinstance(value, datetime.time):
                 return DateTime.Time(value.hour, value.minute, value.second)
             try:
-                stime = time.strptime(value, self.format)
+                if self.format.find(".%f") >= 0:
+                    if '.' in value:
+                        _value = value.split('.')
+                        microseconds = _value[-1]
+                        _l = len(microseconds)
+                        if _l < 6:
+                            _value[-1] = microseconds + '0'*(6 - _l)
+                        elif _l > 6:
+                            _value[-1] = microseconds[:6]
+                        if _l != 6:
+                            value = '.'.join(_value)
+                    else:
+                        value += '.0'
+                value = datetime.datetime.strptime(value, self.format)
+                return DateTime.DateTime(value.year, value.month, value.day,
+                    value.hour, value.minute, value.second)
             except:
                 raise validators.Invalid("expected a date/time string of the '%s' format in the DateTimeCol '%s', got %s %r instead" % \
                     (self.format, self.name, type(value), value), value, state)
-            return DateTime.mktime(stime)
 
         def from_python(self, value, state):
             if value is None:
@@ -1119,7 +1174,7 @@ if mxdatetime_available:
                 (self.name, type(value), value), value, state)
 
 class SODateTimeCol(SOCol):
-    datetimeFormat = '%Y-%m-%d %H:%M:%S'
+    datetimeFormat = '%Y-%m-%d %H:%M:%S.%f'
 
     def __init__(self, **kw):
         datetimeFormat = kw.pop('datetimeFormat', None)
@@ -1250,7 +1305,7 @@ class TimeValidator(DateTimeValidator):
     from_python = to_python
 
 class SOTimeCol(SOCol):
-    timeFormat = '%H:%M:%S'
+    timeFormat = '%H:%M:%S.%f'
 
     def __init__(self, **kw):
         timeFormat = kw.pop('timeFormat', None)
@@ -1307,7 +1362,7 @@ class TimestampCol(Col):
     baseClass = SOTimestampCol
 
 
-class TimedeltaValidator(validators.Validator):
+class TimedeltaValidator(SOValidator):
     def to_python(self, value, state):
         return value
 
@@ -1327,7 +1382,7 @@ class TimedeltaCol(Col):
 
 from decimal import Decimal
 
-class DecimalValidator(validators.Validator):
+class DecimalValidator(SOValidator):
     def to_python(self, value, state):
         if value is None:
             return None
@@ -1452,7 +1507,7 @@ class DecimalStringCol(StringCol):
     baseClass = SODecimalStringCol
 
 
-class BinaryValidator(validators.Validator):
+class BinaryValidator(SOValidator):
     """
     Validator for binary types.
 
@@ -1543,12 +1598,7 @@ class PickleValidator(BinaryValidator):
         if value is None:
             return None
         if isinstance(value, unicode):
-            try:
-                connection = state.connection or state.soObject._connection
-            except AttributeError:
-                dbEncoding = "ascii"
-            else:
-                dbEncoding = getattr(connection, "dbEncoding", None) or "ascii"
+            dbEncoding = self.getDbEncoding(state, default='ascii')
             value = value.encode(dbEncoding)
         if isinstance(value, str):
             return pickle.loads(value)
