@@ -43,8 +43,8 @@ from sresults import SelectResults
 from util.threadinglocal import local
 
 import sys
-if sys.version_info[:3] < (2, 5, 0):
-    raise ImportError, "SQLObject requires Python 2.5.0 or later"
+if sys.version_info[:3] < (2, 6, 0):
+    raise ImportError, "SQLObject requires Python 2.6 or 2.7"
 
 """
 This thread-local storage is needed for RowCreatedSignals. It gathers
@@ -648,8 +648,8 @@ def deprecated(message, level=1, stacklevel=2):
     if warnings_level is not None and warnings_level <= level:
         warnings.warn(message, DeprecationWarning, stacklevel=stacklevel)
 
-if sys.version_info[:3] < (2, 6, 0):
-    deprecated("Support for Python 2.5 has been declared obsolete and will be removed in the next release of SQLObject")
+#if sys.version_info[:3] < (2, 5, 0):
+#    deprecated("Support for Python 2.4 has been declared obsolete and will be removed in the next release of SQLObject")
 
 def setDeprecationLevel(warning=1, exception=None):
     """
